/*
 * Renderer 1. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.scene.util.DrawSceneGraph;

import java.awt.Color;

/**
<pre>{@code
                  y
                  |
                  |
                  | v[4]
                1 +---------------------+ v[5]=(1,1,0)
                 /|                    /|
                / |                   / |
               /  |                  /  |
              /   |                 /   |
             /    |                /    |
       v[7] +---------------------+ v[6]|
            |     |               |     |
            |     |               |     |
            |     | v[0]          |     | v[1]
            |     +---------------|-----+------> x
            |    /                |    /1
            |   /                 |   /
            |  /                  |  /
            | /                   | /
            |/                    |/
          1 +---------------------+
           /v[3]=(0,0,1)          v[2]=(1,0,1)
          /
         /
        z
}</pre>
*/
public class OfflineCubes_v2
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Create a Model object to hold the geometry.
      final Model model = new Model("Cube 1");

      // Add the Model to the Scene twice.
      scene.addPosition(new Position(model, "p1"));
      scene.addPosition(new Position(model, "p2"));

      // Add geometry to the Models.
      model.addVertex(new Vertex(0.0, 0.0, 0.0), // four vertices around the bottom face
                      new Vertex(1.0, 0.0, 0.0),
                      new Vertex(1.0, 0.0, 1.0),
                      new Vertex(0.0, 0.0, 1.0),
                      new Vertex(0.0, 1.0, 0.0), // four vertices around the top face
                      new Vertex(1.0, 1.0, 0.0),
                      new Vertex(1.0, 1.0, 1.0),
                      new Vertex(0.0, 1.0, 1.0));

      model.addPrimitive(new LineSegment(0, 1),  // bottom face
                         new LineSegment(1, 2),  // bottom face
                         new LineSegment(2, 3),  // bottom face
                         new LineSegment(3, 0),  // bottom face
                         new LineSegment(4, 5),  // top face
                         new LineSegment(5, 6),  // top face
                         new LineSegment(6, 7),  // top face
                         new LineSegment(7, 4),  // top face
                         new LineSegment(0, 4),  // back face
                         new LineSegment(1, 5),  // back face
                         new LineSegment(3, 7),  // front face
                         new LineSegment(2, 6)); // front face

      // Push the positions away from where the camera is.
      scene.getPosition(0).translation(0.0,  0.0, -4.0);
      scene.getPosition(1).translation(0.0,  0.0, -4.0);

      // Log information to standard output.
      System.out.println( scene.toString() );

      // Draw a picture of the scene's tree data structure.
      DrawSceneGraph.draw(scene, "SG_OfflineCubes_v2");

      // Create a FrameBuffer to render our Scene into.
      final int width  = 1024;
      final int height = 1024;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.darkGray);

      Rasterize.doClipping = false; // turn off clipping

      for (int j = 0; j <= 100; ++j)
      {
         // Render again.
         fb.clearFB(); // What happens if we don't clear the framebuffer?
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineCubes_v2_Frame%03d.ppm", j));

         // Update the scene by translating each
         // of the positions in camera space.
         final Vector t0 = scene.getPosition(0).getTranslation();
         scene.getPosition(0).translation(t0.x - 0.1,
                                          t0.y - 0.03,
                                          t0.z);
         final Vector t1 = scene.getPosition(1).getTranslation();
         scene.getPosition(1).translation(t1.x + 0.1,
                                          t1.y,
                                          t1.z);
      }
   }
}
